@@ -2,12 +2,6 @@
 
 """
 Quartet approach for best match inference.
-
-Methods in this module:
-    - quartet_qinfer
-    - quartet_from_scenario
-Classes:
-    - Quartets
 """
 
 import itertools, random,  subprocess, time, os
@@ -15,12 +9,11 @@ import itertools, random,  subprocess, time, os
 import numpy as np
 import networkx as nx
 
-from asymmetree.best_matches import TrueBMG
-from asymmetree.tools import FileIO
+from asymmetree.tools.GraphTools import symmetric_part
+from asymmetree.file_io.ScenarioFileIO import parse_bmg_edges, matrix_to_phylip, species_to_genes, write_newick
 
 
-__author__ = "David Schaller"
-__copyright__ = "Copyright (C) 2019, David Schaller"
+__author__ = 'David Schaller'
 
 
 # --------------------------------------------------------------------------
@@ -37,7 +30,7 @@ class Quartets:
     """Implementation of the Quartet approach for best match inference."""
     
     def __init__(self, scenario, distance_matrix,
-                 voting_mode="majority",
+                 voting_mode='majority',
                  use_distant_genes=False,
                  insecurity_factor=0.0,
                  y_candidates=None,
@@ -79,6 +72,7 @@ class Quartets:
     
     
     def compute_lcas_in_S(self):
+        
         self.S.supply_leaves()
         N = len(self.S.root.leaves)
         self.S_leaf_index = {leaf.ID: i for i, leaf in enumerate(self.S.root.leaves)}
@@ -106,6 +100,7 @@ class Quartets:
     
     
     def compute_outgroup_species(self):
+        
         self.lca_to_outgroups = {key: set() for key in self.subtree_species}
         
         root_done = False
@@ -288,9 +283,9 @@ class Quartets:
         if self.use_distant_genes:
             self.build_matrix_I()  
         
-        self.BMG = nx.DiGraph()
+        self.bmg = nx.DiGraph()
         for g in self.genes:
-            self.BMG.add_node(g.ID, label=g.label, color=g.color)
+            self.bmg.add_node(g.ID, label=g.label, color=g.color)
         
         # consider all genes of other color as potential best matches
         if self.y_candidates is None:
@@ -303,11 +298,11 @@ class Quartets:
                     if species == x.color:
                         continue
                     elif len(Y) == 1:
-                        self.BMG.add_edge(x.ID, Y[0].ID)
+                        self.bmg.add_edge(x.ID, Y[0].ID)
                     elif len(Y) > 1:
                         best_matches = self.find_best_matches(x, Y, Z)
                         for y in best_matches:
-                            self.BMG.add_edge(x.ID, y.ID)
+                            self.bmg.add_edge(x.ID, y.ID)
                             
         # consider only a given subset as best matches
         else:
@@ -324,15 +319,15 @@ class Quartets:
                         j += 1
                     if self.subtree_index[x.color] == self.subtree_index[Y[i].color]:
                         if i == j-1:
-                            self.BMG.add_edge(x.ID, Y[i].ID)
+                            self.bmg.add_edge(x.ID, Y[i].ID)
                         else:
                             best_matches = self.find_best_matches(x, Y[i:j], Z)
                             for y in best_matches:
-                                self.BMG.add_edge(x.ID, y.ID)
+                                self.bmg.add_edge(x.ID, y.ID)
                     i = j
         
-        self.RBMG = TrueBMG.RBMG_from_BMG(self.BMG)
-        return self.BMG, self.RBMG
+        self.rbmg = symmetric_part(self.bmg)
+        return self.bmg, self.rbmg
     
     
     def build_graphs_closest_outgroups(self):
@@ -341,9 +336,9 @@ class Quartets:
         self.compute_outgroup_species()
         self.build_matrix_I()
         
-        self.BMG = nx.DiGraph()
+        self.bmg = nx.DiGraph()
         for g in self.genes:
-            self.BMG.add_node(g.ID, label=g.label, color=g.color)
+            self.bmg.add_node(g.ID, label=g.label, color=g.color)
         
         # consider all genes of other color as potential best matches
         if self.y_candidates is None:
@@ -354,12 +349,12 @@ class Quartets:
                     if species == x.color:
                         continue
                     elif len(Y) == 1:
-                        self.BMG.add_edge(x.ID, Y[0].ID)
+                        self.bmg.add_edge(x.ID, Y[0].ID)
                     elif len(Y) > 1:
                         Z = self.get_outgroups_closest(x, Y[0].color)
                         best_matches = self.find_best_matches(x, Y, Z)
                         for y in best_matches:
-                            self.BMG.add_edge(x.ID, y.ID)
+                            self.bmg.add_edge(x.ID, y.ID)
                             
         # consider only a given subset as best matches
         else:
@@ -375,16 +370,16 @@ class Quartets:
                         j += 1
                     if self.subtree_index[x.color] == self.subtree_index[Y[i].color]:
                         if i == j-1:
-                            self.BMG.add_edge(x.ID, Y[i].ID)
+                            self.bmg.add_edge(x.ID, Y[i].ID)
                         else:
                             Z = self.get_outgroups_closest(x, Y[i].color)
                             best_matches = self.find_best_matches(x, Y[i:j], Z)
                             for y in best_matches:
-                                self.BMG.add_edge(x.ID, y.ID)
+                                self.bmg.add_edge(x.ID, y.ID)
                     i = j
                     
-        self.RBMG = TrueBMG.RBMG_from_BMG(self.BMG)
-        return self.BMG, self.RBMG
+        self.rbmg = symmetric_part(self.bmg)
+        return self.bmg, self.rbmg
     
     
     def build_graphs(self):
@@ -429,7 +424,7 @@ def quartet_qinfer(scenario,
     elif os.path.exists(binary_path):
         qinfer_command = binary_path
     else:
-        raise FileNotFoundError("Path to qinfer binary file '{}' does not exist!".format(binary_path))
+        raise FileNotFoundError("path to qinfer binary file '{}' does not exist".format(binary_path))
     
     output = -1
     
@@ -448,17 +443,17 @@ def quartet_qinfer(scenario,
     try:
         output = subprocess.run(command, stdout=subprocess.PIPE)
     except:
-        raise FileNotFoundError("Calling qinfer failed!")
+        raise FileNotFoundError("calling qinfer failed")
         
     exec_time = time.time() - start
     
     if output == -1:
-        raise Exception("No output from qinfer!")
+        raise Exception("no output from qinfer")
     
-    BMG = FileIO.parse_BMG_edges(output.stdout.decode(), scenario)
-    RBMG = TrueBMG.RBMG_from_BMG(BMG)
+    bmg = parse_bmg_edges(output.stdout.decode(), scenario)
+    rbmg = symmetric_part(bmg)
     
-    return BMG, RBMG, exec_time
+    return bmg, rbmg, exec_time
 
 
 def quartet_from_scenario(scenario, epsilon=-1,
@@ -484,11 +479,11 @@ def quartet_from_scenario(scenario, epsilon=-1,
     tree_filename = "temp_tree.txt"
     
     matrix = scenario.get_distance_matrix()
-    FileIO.matrix_to_phylip(matrix_filename, scenario.genes, matrix)
-    FileIO.species_to_genes(species_filename, scenario)
-    FileIO.write_newick(tree_filename, scenario.S)
+    matrix_to_phylip(matrix_filename, scenario.genes, matrix)
+    species_to_genes(species_filename, scenario)
+    write_newick(tree_filename, scenario.S)
     
-    BMG, RBMG, exec_time = quartet_qinfer(scenario,
+    bmg, rbmg, exec_time = quartet_qinfer(scenario,
                                           matrix_filename, species_filename, tree_filename,
                                           epsilon=epsilon,
                                           closest_outgroups=closest_outgroups,
@@ -499,4 +494,4 @@ def quartet_from_scenario(scenario, epsilon=-1,
     os.remove(species_filename)
     os.remove(tree_filename)
     
-    return BMG, RBMG, exec_time
\ No newline at end of file
+    return bmg, rbmg, exec_time
\ No newline at end of file
