@@ -18,11 +18,10 @@ from collections import deque
 
 from asymmetree.tools.Tree import Tree, TreeNode
 from asymmetree.tools.DoublyLinkedList import DLList
-from asymmetree.tools.hdtgraph.DynamicGraph import Graph as HDTGraph
+from asymmetree.tools.hdtgraph.DynamicGraph import HDTGraph
 
 
-__author__ = "David Schaller"
-__copyright__ = "Copyright (C) 2018, David Schaller"
+__author__ = 'David Schaller'
 
 
 class BuildST:
@@ -478,16 +477,16 @@ class ConnectedComp_simple:
         return [node for node in self.component.keys() if isinstance(node, XpNode)]
 
 
-if __name__ == "__main__":
+if __name__ == '__main__':
     
     # ----- TESTING THIS MODULE -----
     
     import numpy as np
     from asymmetree.tools.PhyloTree import PhyloTree
-    from asymmetree.simulator.TreeSimulator import observable_tree
+    from asymmetree.treeevolve import observable_tree
     
     N = 20
-    t = PhyloTree.random_colored_tree(N, [1,2,3], binary=False)
+    t = PhyloTree.random_colored_tree(N, 3, binary=False)
     print(t.to_newick(color=False, distance=False, label_inner=False))
     
     partial_trees = []
@@ -499,7 +498,7 @@ if __name__ == "__main__":
         for v in t.preorder():
             if not v.children:
                 if j in mark:
-                    v.label = "*"
+                    v.label = '*'
                 else:
                     v.label = str(v.ID)
                 j += 1
@@ -515,4 +514,5 @@ if __name__ == "__main__":
         
     print(supertree.to_newick())
     
+    # randomly True or False
     print(t.compare_topology(supertree))
\ No newline at end of file
