@@ -2,19 +2,16 @@
 
 """
 Tree data structure for phylogentic trees.
-
-Classes in this module:
-    - PhyloTreeNode
-    - PhyloTree
 """
 
-import collections, random, re
+import collections, itertools, random, re, pickle, json, os
+import numpy as np
 import networkx as nx
 
 from asymmetree.tools.Tree import Tree, TreeNode
 
-__author__ = "David Schaller"
-__copyright__ = "Copyright (C) 2020, David Schaller"
+
+__author__ = 'David Schaller'
 
 
 class PhyloTreeNode(TreeNode):
@@ -26,29 +23,35 @@ class PhyloTreeNode(TreeNode):
     __slots__ = ('color', 'dist', 'tstamp', 'transferred')
     
     
-    def __init__(self, ID, label="", color=None,
+    def __init__(self, ID, label='', color=None,
                  dist = 1.0, tstamp=None, transferred=0):
         
         super().__init__(ID, label=label)
-        self.color = color
-        self.dist = dist
-        self.tstamp = tstamp
-        self.transferred = transferred
+        self.color = color                      # color / reconciliation
+        self.dist = dist                        # distance from parent
+        self.tstamp = tstamp                    # time stamp
+        self.transferred = transferred          # 1 if edge parent-self was an 
+                                                # HGT edge, and 0 otherwise
 
     
     def __repr__(self):
         
-        return "pn" + str(self.ID)
+        return '<PhyloTN: {}>'.format(self.ID)
     
     
     def __str__(self):
         
-        if isinstance(self.color, tuple):
-            return "{}<{}-{}>:{}".format(self.label, *self.color, self.dist)
+        if isinstance(self.color, (tuple, list)):
+            return '{}<{}-{}>:{}'.format(self.label, *self.color, self.dist)
         elif self.color:
-            return "{}<{}>:{}".format(self.label, self.color, self.dist)
+            return '{}<{}>:{}'.format(self.label, self.color, self.dist)
         else:
-            return "{}:{}".format(self.label, self.dist)
+            return '{}:{}'.format(self.label, self.dist)
+        
+    
+    def is_loss(self):
+        
+        return self.label == '*'
 
 
 class PhyloTree(Tree):
@@ -59,8 +62,8 @@ class PhyloTree(Tree):
     
     def __init__(self, root):
         
-        if not isinstance(root, PhyloTreeNode):
-            raise TypeError("root must be of type 'PhyloTreeNode'.")
+        if root is not None and not isinstance(root, PhyloTreeNode):
+            raise TypeError("root must be of type 'PhyloTreeNode'")
         super().__init__(root)
     
     
@@ -110,30 +113,111 @@ class PhyloTree(Tree):
             
             contracted.add(v)
             
+            
+    def remove_planted_root(self):
+        """Removes the planted root if existent.
+        
+        The the tree has a planted root, its single child becomes the new root.
+        """
+        
+        if len(self.root.children) == 1:
+        
+            new_root = self.root.children[0]
+            new_root.detach()
+            self.root = new_root
+            new_root.dist = 0.0
+            
     
-    def supply_leaves(self, node=None, exclude_losses=True):
-        """Add the leaves to all nodes that are in the subtree of a specific node."""
+    def supply_leaves(self, exclude_losses=True):
+        """Add the leaves to all nodes under their respective subtrees."""
+        
+        def _supply_leaves(node, exclude_losses):
         
-        if not node:
-            self.supply_leaves(self.root, exclude_losses=exclude_losses)
-            return self.root.leaves
-        else:
             node.leaves = []
+            
             if not node.children:
-                if exclude_losses and node.label != "*":
+                if exclude_losses and not node.is_loss():
                     node.leaves.append(node)
                 elif not exclude_losses:
                     node.leaves.append(node)
             else:
                 for child in node.children:
-                    node.leaves.extend(self.supply_leaves(child, exclude_losses=exclude_losses))
+                    node.leaves.extend(_supply_leaves(child, exclude_losses))
+                    
             return node.leaves
+        
+        
+        if self.root:
+            return _supply_leaves(self.root, exclude_losses)
+        else:
+            return []
+    
+    
+    def color_sorted_leaves(self):
+        
+        self.supply_leaves()
+        color_dict = {}
+        
+        for leaf in self.root.leaves:
+            if leaf.color not in color_dict:
+                color_dict[leaf.color] = []
+            color_dict[leaf.color].append(leaf)
+            
+        leaves = []
+        for color, leaf_list in color_dict.items():
+            for leaf in leaf_list:
+                leaves.append(leaf)
+        
+        return leaves
+    
+    
+    def distance_matrix(self, leaf_order=None):
+        """Computes the distance matrix from the phylogenetic tree.
+        
+        Additionally a list of leaves corresponding to the indices in the matrix
+        is returned.
+        
+        Keyword arguments:
+            leaf_order -- list of leaves defining the indices for the matrix;
+                default is None, in which case leaves are indexed in sibling order.
+        """
+        
+        distance_dict, _ = self.distances_from_root()
+        self.supply_leaves()
+        
+        if leaf_order:
+            if set(leaf_order) != set(self.root.leaves):
+                raise ValueError('ordered leaf list does not match with the '\
+                                 'leaves in the tree')
+            leaves = leaf_order
+        else:
+            # leaves in sibling order
+            leaves = self.root.leaves      
+        
+        leaf_index = {l: i for i, l in enumerate(leaves)}
+        
+        D = np.zeros((len(leaves),len(leaves)), dtype=np.float)
+        
+        for v in self.preorder():
+            if v.children:
+                for c1, c2 in itertools.combinations(v.children, 2):
+                    for x in c1.leaves:
+                        x_index = leaf_index[x]
+                        x_dist = distance_dict[x] - distance_dict[v]
+                        for y in c2.leaves:
+                            y_index = leaf_index[y]
+                            y_dist = distance_dict[y] - distance_dict[v]
+                            D[x_index, y_index] = x_dist + y_dist
+                            D[y_index, x_index] = x_dist + y_dist
+        
+        return leaves, D
     
     
     def distances_from_root(self):
         
         leaf_distances = []
         distance_dict = {}
+        
         for v in self.preorder():
             if not v.parent:
                 distance_dict[v] = 0.0
@@ -142,6 +226,7 @@ class PhyloTree(Tree):
                 distance_dict[v] = depth
             if not v.children:
                 leaf_distances.append( (str(v), distance_dict[v]) )
+                
         return distance_dict, leaf_distances
     
     
@@ -163,43 +248,66 @@ class PhyloTree(Tree):
         
         return T
     
+    
+    def count_node_types(self):
+        
+        counts = {'S': 0, 'D': 0, 'L': 0, 'H': 0, 'extant': 0}
+        
+        for v in self.preorder():
+            
+            if not v.children:
+                if v.is_loss():
+                    counts['L'] += 1
+                else:
+                    counts['extant'] += 1
+            
+            elif v.label == 'S':
+                counts['S'] += 1
+            elif v.label == 'D':
+                counts['D'] += 1
+            elif v.label == 'H':
+                counts['H'] += 1
+        
+        return counts        
+    
 
 # --------------------------------------------------------------------------
 #                          TREE  <--->  NEWICK
 # --------------------------------------------------------------------------
         
-    def to_newick(self, node=None, 
-                  label=True, color=True, distance=True,
-                  label_inner=True, color_inner=False):
+    def to_newick(self, label=True, color=True, distance=True,
+                        label_inner=True, color_inner=False):
         """Recursive PhyloTree --> Newick (str) function."""
         
-        if node is None:
-            return self.to_newick(self.root,
-                                  label=label, color=color, distance=distance,
-                                  label_inner=label_inner, color_inner=color_inner) + ";"
-        elif not node.children:
-            token = ""
-            if label:
-                token += str(node.label)
-            if color and node.color:
-                token += "<{}-{}>".format(*node.color) if isinstance(node.color, tuple) else "<{}>".format(node.color)
-            if distance:
-                token += ":{}".format(node.dist)
-            return token
+        def _to_newick(node):
+            
+            if not node.children:
+                token = ''
+                if label:
+                    token += str(node.label)
+                if color and node.color:
+                    token += '<{}-{}>'.format(*node.color) if isinstance(node.color, (tuple, list)) else '<{}>'.format(node.color)
+                if distance:
+                    token += ":{}".format(node.dist)
+                return token
+            else:
+                s = ''
+                for child in node.children:
+                    s += _to_newick(child) + ','
+                token = ''
+                if label and label_inner:
+                    token += str(node.label)
+                if color_inner and node.color:
+                    token += '<{}-{}>'.format(*node.color) if isinstance(node.color, (tuple, list)) else '<{}>'.format(node.color)
+                if distance:
+                    token += ':{}'.format(node.dist)
+                return '({}){}'.format(s[:-1], token)
+        
+        
+        if self.root:
+            return _to_newick(self.root) + ';'
         else:
-            s = ''
-            for child in node.children:
-                s += self.to_newick(node=child,
-                                    label=label, color=color, distance=distance,
-                                    label_inner=label_inner, color_inner=color_inner) + ","
-            token = ""
-            if label and label_inner:
-                token += str(node.label)
-            if color_inner and node.color:
-                token += "<{}-{}>".format(*node.color) if isinstance(node.color, tuple) else "<{}>".format(node.color)
-            if distance:
-                token += ":{}".format(node.dist)
-            return "({}){}".format(s[:-1], token)
+            return ';'
     
     
     @staticmethod
@@ -227,7 +335,7 @@ class PhyloTree(Tree):
                 if child[0] == '(':                                 # the child has subtrees
                     end = child.rfind(')')
                     if end == -1:
-                        raise ValueError("Invalid Newick-String!")
+                        raise ValueError('invalid Newick string')
                     parse_subtree(node, child[1:end])               # recursive call 'parse_subtree'
                 child = child[end+1:].strip()
                 label_col_dist = label_col_dist_regex.match(child)
@@ -265,7 +373,7 @@ class PhyloTree(Tree):
                     current += c
                 elif c == ')':
                     if stack <= 0:
-                        raise ValueError("Invalid Newick-String!")
+                        raise ValueError('invalid Newick string')
                     stack -= 1
                     current += c
                 else:
@@ -274,7 +382,7 @@ class PhyloTree(Tree):
             return children
         
         if not isinstance(newick, str):
-            raise ValueError("Newick parser needs a 'str' as input.")
+            raise ValueError("Newick parser needs a 'str' as input")
         end = newick.find(";")
         if end != -1:
             newick = newick[:end]
@@ -287,7 +395,7 @@ class PhyloTree(Tree):
                                             # (important for non-recursive to_newick2)
             return PhyloTree(root)
         else:
-            raise ValueError("Invalid Newick-String!")
+            raise ValueError('invalid Newick string')
     
     
     def reconstruct_IDs(self):
@@ -327,17 +435,24 @@ class PhyloTree(Tree):
             
     def to_nx(self):
         
-        self.check_integrity()
+        self._assert_integrity()
         G = nx.DiGraph()
         
-        for u, v in self.edges():
-            if u.ID not in G:
-                G.add_node(u.ID, label=u.label, color=u.color, tstamp=u.tstamp)
-            if v.ID not in G:
-                G.add_node(v.ID, label=v.label, color=v.color, tstamp=v.tstamp)
+        if not self.root:
+            return G, None
+        
+        for v in self.preorder():
+            G.add_node(v.ID, label=v.label,
+                       color=v.color,
+                       tstamp=v.tstamp,
+                       dist=v.dist,
+                       transferred=v.transferred)
+        
+        for u, v, sibling_nr in self.edges_sibling_order():
             if u is v or u.ID == v.ID:
-                print("Loop at", str(u), str(v), v.transferred)
-            G.add_edge(u.ID, v.ID, dist=v.dist, transferred=v.transferred)
+                raise RuntimeError('loop at {} and {}'.format(u, v))
+            G.add_edge(u.ID, v.ID)
+            G.nodes[v.ID]['sibling_nr'] = sibling_nr
             
         return G, self.root.ID
     
@@ -346,12 +461,21 @@ class PhyloTree(Tree):
     def parse_nx(G, root):
         
         number_of_leaves = 0
+        
+        if root is None:
+            return PhyloTree(None)
     
         def build_tree(ID, parent=None):
+            
             nonlocal number_of_leaves
+            
             treenode = PhyloTreeNode(ID, label=G.nodes[ID]['label'])
+            
             if parent:
                 parent.add_child(treenode)
+                
+                # edge attributes are deprecated,
+                # for compatibilty with older versions of AsymmeTree:
                 if 'dist' in G.edges[parent.ID, ID]:
                     treenode.dist = G.edges[parent.ID, ID]['dist']
                 if 'transferred' in G.edges[parent.ID, ID]:
@@ -360,25 +484,117 @@ class PhyloTree(Tree):
             if 'label' in G.nodes[ID]:
                 treenode.label = G.nodes[ID]['label']
             if 'color' in G.nodes[ID]:
-                treenode.color = G.nodes[ID]['color']
+                if isinstance(G.nodes[ID]['color'], list):
+                    treenode.color = tuple(G.nodes[ID]['color'])
+                else:
+                    treenode.color = G.nodes[ID]['color']
             if 'tstamp' in G.nodes[ID]:
                 treenode.tstamp = G.nodes[ID]['tstamp']
+            if 'dist' in G.nodes[ID]:
+                treenode.dist = G.nodes[ID]['dist']
+            if 'transferred' in G.nodes[ID]:
+                treenode.transferred = G.nodes[ID]['transferred']
             
-            for child_id in G.neighbors(ID):
+            try:
+                children_ids = sorted(G.neighbors(ID),
+                                  key=lambda item: G.nodes[item]['sibling_nr'])
+            except KeyError:
+                # for compatibilty with older versions of AsymmeTree:
+                children_ids = [item for item in G.neighbors(ID)]
+            
+            for child_id in children_ids:
                 build_tree(child_id, parent=treenode)
             if G.out_degree(ID) == 0:
                 number_of_leaves += 1
             
             return treenode
+        
         tree = PhyloTree(build_tree(root))
         tree.number_of_species = number_of_leaves
         
         return tree
-# --------------------------------------------------------------------------
 
+# --------------------------------------------------------------------------
+#                           SERIALIZATION
+# --------------------------------------------------------------------------
+    
+    @staticmethod
+    def _infer_serialization_mode(filename):
+        
+        _, file_ext = os.path.splitext(filename)
+        
+        if file_ext in ('.json', '.JSON'):
+            return 'json'
+        elif file_ext in ('.pickle', '.PICKLE'):
+            return 'pickle'
+        else:
+            raise ValueError('serialization format is not supplied and could '\
+                             'not be inferred from file extension')
+            
+            
+    def serialize(self, filename, mode=None):
+        """Serialize the tree using pickle."""
+        
+        if not mode:
+            mode = PhyloTree._infer_serialization_mode(filename)
+        
+        tree_nx, root_id = self.to_nx()
+        
+        if mode == 'json':
+            data = nx.readwrite.json_graph.tree_data(tree_nx, root=root_id)
+            
+            with open(filename, 'w') as f:
+                f.write( json.dumps(data) )
+                
+        elif mode == 'pickle':
+            pickle.dump( (tree_nx, root_id), open(filename, "wb") )
+            
+        else:
+            raise ValueError("serialization mode '{}' not supported".format(mode))
+    
+    
+    @staticmethod
+    def load(filename, mode=None):
+        """Load a phylogenetic tree from a file.
+        
+        Using either the Python module json or pickle."""
+        
+        if not mode:
+            mode = PhyloTree._infer_serialization_mode(filename)
+        
+        if mode == 'json':
+            with open(filename, 'r') as f:
+                data = json.loads( f.read() )
+                
+            tree_nx = nx.readwrite.json_graph.tree_graph(data)
+            
+            root_id = None
+            for v in tree_nx:
+                if tree_nx.in_degree(v) == 0:
+                    root_id = v
+                    break
+                
+            if root_id is None:
+                raise RuntimeError('could not identify root')
+                
+        elif mode == 'pickle':
+            tree_nx, root_id = pickle.load( open(filename, "rb") )
+            
+        else:
+            raise ValueError("serialization mode '{}' not supported".format(mode))
+        
+        tree = PhyloTree.parse_nx(tree_nx, root_id)
+        
+        return tree
+    
+# --------------------------------------------------------------------------
+        
     
     def copy(self):
         
+        if not self.root:
+            return PhyloTree(None)
+        
         orig_to_new = {}
         
         for orig in self.preorder():
@@ -406,12 +622,17 @@ class PhyloTree(Tree):
             binary - forces the tree to be binary; default is False
         """
         
-        if not (isinstance(N, int) and isinstance(colors, collections.Iterable)):
-            raise TypeError("N must be of type 'int' and colors must be iterable!")
-        root = PhyloTreeNode(0, label="0")
+        if not isinstance(N, int):
+            raise TypeError("N must be of type 'int'")
+            
+        if isinstance(colors, int):
+            colors = [i+1 for i in range(colors)]
+        elif not isinstance(colors, collections.abc.Iterable):
+            raise TypeError("'colors' must be of type 'int' or iterable")
+        
+        root = PhyloTreeNode(0, label='0')
         tree = PhyloTree(root)
         node_list = [root]
-        break_prob = [0.5, 0.5]
         nr, leaf_count = 1, 1
         
         while leaf_count < N:
@@ -428,14 +649,6 @@ class PhyloTree(Tree):
                 leaf_count += 1
             elif node.children and not binary:
                 # add only one child if there are already children
-                
-                # probability to add another child (decraeses exponentially)
-                break_prob[0] = 1 / 1.4**len(node.children)
-                # probability to choose another node
-                break_prob[1] = 1 - break_prob[0]
-                if random.choices( (0,1), weights=break_prob )[0] == 1:
-                    continue
-                
                 new_child = PhyloTreeNode(nr, label=str(nr))
                 node.add_child(new_child)
                 node_list.append(new_child)
@@ -447,33 +660,46 @@ class PhyloTree(Tree):
                 node.color = random.choice(colors)
                 
         return tree
-    
-    
-    def check_integrity(self):
-        
-        for v in self.preorder():
-            for child in v.children:
-                if child is v:
-                    print("Loop at " + str(v))
-                    raise KeyboardInterrupt
-                if child.parent is not v:
-                    print("PhyloTree invalid for " + str(v) + " and " + str(child))
-                    raise KeyboardInterrupt
+   
+     
+# --------------------------------------------------------------------------
+#                         TREE MANIPULATION
+# -------------------------------------------------------------------------- 
 
+def delete_losses_and_contract(tree, inplace=False):
     
-if __name__ == "__main__":
-    colors = ("s", "t", "v", "w")
-    N = 20
+    if not inplace:
+        tree = tree.copy()
     
-    t = PhyloTree.random_colored_tree(N, colors)
-    print("------------- Random tree test -------------")
-    print( t.to_newick() )
-    print("--------------------------------------------")
+    loss_nodes = []
+    for node in tree.postorder():
+        if not node.children and node.is_loss():
+            loss_nodes.append(node)
     
-    t2 = PhyloTree.parse_newick(t.to_newick())
-    print( t2.to_newick() )
+    # traverse from loss node to root delete if degree <= 1
+    for loss_node in loss_nodes:
+        current = tree.delete_and_reconnect(loss_node,
+                                            add_distances=True,
+                                            keep_transferred=True)
+        
+        while len(current.children) < 2 and current.parent:
+            current = tree.delete_and_reconnect(current,
+                                                add_distances=True,
+                                                keep_transferred=True)
     
-    nx_tree, nx_root = t.to_nx()
-    t3 = PhyloTree.parse_nx(nx_tree, nx_root)
-    print("--------------------------------------------")
-    print( t3.to_newick() )
+    return tree
+
+
+def remove_planted_root(tree, inplace=True):
+    
+    if not inplace:
+        tree = tree.copy()
+        
+    # delete the root if the tree is planted
+    tree.remove_planted_root()
+        
+    if not tree.root.children and not tree.root.label:
+        # no surviving genes --> return empty tree
+        tree.root = None
+    
+    return tree
\ No newline at end of file
