@@ -15,8 +15,7 @@ References:
 from asymmetree.tools.DoublyLinkedList import DLList
 
 
-__author__ = "David Schaller"
-__copyright__ = "Copyright (C) 2018, David Schaller"
+__author__ = 'David Schaller'
 
 
 class DGNode:
